@@ -35,10 +35,16 @@ using Gtk;
 
 using Beagle.Util;
 
+#if ENABLE_WEBSVC
+using Beagle.websvc;
+using MA=Mono.ASPNET;
+#endif
 
 namespace Beagle.Daemon {
 	class BeagleDaemon {
 
+		public static Thread MainLoopThread = null;
+
 		private static FactoryImpl factory = null;
 
 #if ENABLE_NETWORK
@@ -110,13 +116,6 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
-			
-			// FIXME: This try/catch is to work around a bug in mono 1.1.3
-			try {
-				Logger.Log.Debug ("Command Line: {0}",
-						  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
-			} catch (Exception ex) { }
 
 		}
 
@@ -159,31 +158,48 @@ namespace Beagle.Daemon {
 		{
 			while (! Shutdown.ShutdownRequested) {
 				int vm_size = SystemInformation.VmSize;
+				int vm_rss = SystemInformation.VmRss;
 
-				Logger.Log.Debug ("Memory usage: VmSize={0}  GC.GetTotalMemory={1}",
-						  vm_size, GC.GetTotalMemory (false));
+				Logger.Log.Debug ("Memory usage: VmSize={0:.0} MB, VmRSS={1:.0} MB,  GC.GetTotalMemory={2}",
+						  vm_size/1024.0, vm_rss/1024.0, GC.GetTotalMemory (false));
 
-				if (vm_size > 200 * 1024) {
+				if (vm_size > 300 * 1024) {
 					Logger.Log.Debug ("VmSize too large --- shutting down");
 					Shutdown.BeginShutdown ();
 				}
 
-				Thread.Sleep (1000);
+				Thread.Sleep (5000);
 			}
 		}
 
+
+#if ENABLE_WEBSVC
+		static Mono.ASPNET.ApplicationServer appServer = null;
+		const string DEFAULT_XSP_ROOT="/usr/local/share/doc/xsp/test";
+		const string DEFAULT_XSP_PORT = "8888";
+		static string[] xsp_param = {"--port", "8888", "--root", DEFAULT_XSP_ROOT, 
+			"--applications", "/:" + DEFAULT_XSP_ROOT + ",/beagle:" + DEFAULT_XSP_ROOT + "/beagle", "--nonstop"};
+#endif 
 		public static int Main (string[] args)
 		{
 			// Process the command-line arguments
 
 			bool arg_replace = false;
 			bool arg_debug = false;
-			bool arg_debug_inotify = false;
 			bool arg_debug_memory = false;
-			bool arg_network = false;
 			bool arg_fg = false;
 			bool arg_disable_scheduler = false;
+#if ENABLE_NETWORK
+			bool arg_network = false;
 			int arg_port = 0;
+#endif
+
+#if ENABLE_WEBSVC
+			bool web_global = false;
+			bool web_start = false;
+			string web_port = DEFAULT_XSP_PORT;
+			string web_rootDir = DEFAULT_XSP_ROOT;
+#endif
 
 			int i = 0;
 			while (i < args.Length) {
@@ -212,11 +228,6 @@ namespace Beagle.Daemon {
 					arg_debug = true;
 					break;
 
-				case "--debug-inotify":
-					arg_debug = true;
-					arg_debug_inotify = true;
-					break;
-
 				case "--debug-memory":
 					arg_debug = true;
 					arg_debug_memory = true;
@@ -234,14 +245,15 @@ namespace Beagle.Daemon {
 					++i; // we used next_arg
 					break;
 
-				case "--enable-network":
-					arg_network = true;
-					break;
-
 				case "--disable-scheduler":
 					arg_disable_scheduler = true;
 					break;
 
+#if ENABLE_NETWORK
+				case "--enable-network":
+					arg_network = true;
+					break;
+
 				case "--port":
 					try {
 						arg_port = Convert.ToInt32(next_arg);
@@ -250,7 +262,29 @@ namespace Beagle.Daemon {
 					}
 					++i; // we used next_arg
 					break;
+#endif
 
+#if ENABLE_WEBSVC
+				case "--web-global":
+					web_global = true;
+					break;
+
+				case "--web-start":
+					web_start = true;
+					break;
+
+				case "--web-port":
+					web_port = next_arg;
+					++i; 
+					web_start = true;
+					break;
+
+				case "--web-root":
+					web_rootDir = next_arg;
+					++i; 
+					web_start = true;
+					break;
+#endif 
 				default:
 					Console.WriteLine ("Ignoring unknown argument '{0}'", arg);
 					break;
@@ -258,6 +292,8 @@ namespace Beagle.Daemon {
 				}
 			}
 
+			MainLoopThread = Thread.CurrentThread;
+
 			// Initialize logging.
 			// If we saw the --debug arg, set the default logging level
 			// accordingly.
@@ -265,24 +301,23 @@ namespace Beagle.Daemon {
 			if (arg_debug)
 				Logger.DefaultLevel = LogLevel.Debug;
 
-			if (arg_debug_inotify)
-				Inotify.Log.Level = LogLevel.Debug;
-			else {
-				// FIXME: A hard-wired constant for Inotify.Log doesn't really
-				// belong here.
-				// This overrides the default, so this logger is "decoupled".
-				Inotify.Log.Level = LogLevel.Warn;
-			}
+			Logger.LogToFile (PathFinder.LogDir, "Beagle", arg_fg);
+
+			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
+			
+			// FIXME: This try/catch is to work around a bug in mono 1.1.3
+			try {
+				Logger.Log.Debug ("Command Line: {0}",
+						  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
+			} catch (Exception ex) { }
 
-			bool echo = false;
-			if (arg_fg) {
-				// If we are running in the foreground, we want to echo the log to stdout.
-				echo = true;
-			}
-			SetupLog (echo);
 
 			// Make sure that extended attributes can be set.  If not, bail out with a message.
-			if (! ExtendedAttribute.Test (PathFinder.RootDir)) {
+			// FIXME FIXME FIXME: This assumes that EAs work the same on your storage dir
+			// as they do on your home dir, which is obviously not a good assumption.
+			// We just need to pick a file under the home dir with the right permissions
+			// and test it.
+			if (! ExtendedAttribute.Test (PathFinder.StorageDir)) {
 				Logger.Log.Fatal ("Could not set extended attributes on a file in your home "
 						  + "directory.  See "
 						  + "http://www.beaglewiki.org/index.php/Enable%20Extended%20Attributes "
@@ -299,7 +334,7 @@ namespace Beagle.Daemon {
 			try {
 				Logger.Log.Debug ("Initializing D-BUS");
 				DBusisms.Init ();
-				Application.Init ();
+				Application.InitCheck ("beagled", ref args);
 
 				Logger.Log.Debug ("Acquiring {0} D-BUS service", Beagle.DBusisms.Name);
 				if (!DBusisms.InitService (Beagle.DBusisms.Name)) {
@@ -317,7 +352,7 @@ namespace Beagle.Daemon {
 				Logger.Log.Fatal ("Couldn't connect to the session bus.  "
 						  + "See http://beaglewiki.org/index.php/Installing%20Beagle "
 						  + "for information on setting up a session bus.");
-				Logger.Log.Fatal (e);
+				Logger.Log.Fatal (e.Message);
 				return 1;
 			} catch (Exception e) {
 				Logger.Log.Fatal ("Could not initialize Beagle's bus connection.");
@@ -362,14 +397,42 @@ namespace Beagle.Daemon {
 					Logger.Log.Error ("Could not initialize network service"); 
 				}
 			}
-
 #endif
 
+#if ENABLE_WEBSVC		
+			//Beagle Web, WebService access initialization code:
+			string msg = "Started beagledWeb & beagledWebSvc Listener. Internal Web Server NOT started.";
+
+			xsp_param[1] = web_port;
+			xsp_param[3] = web_rootDir;
+			if (web_start)	
+			{
+				//Start beagled internal web server (bgXsp)
+				int ret = Mono.ASPNET.Server.initXSP(xsp_param, out appServer);
+				msg = "Started beagledWeb & beagledWebSvc Listener \n";
+				if (ret == 0)
+					msg += "Internal Web Server started";
+				else
+					msg += "Error starting Internal Web Server";
+			}	
+
+			//start web-access server first
+			beagledWeb.init (web_global);
+
+			//Next start web-service server 
+			beagledWebSvc.init (web_global);
+
+			//Console.WriteLine (msg);
+			Logger.Log.Debug (msg);
+
+			msg = "Global WebAccess " + (web_global ? "Enabled":"Disabled");
+			//Console.WriteLine (msg);
+			Logger.Log.Debug (msg);
+#endif
 			// Set up out-of-process indexing
 			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
 				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
 
-
 			// Start the query driver.
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
@@ -383,7 +446,7 @@ namespace Beagle.Daemon {
 			// Start our Inotify threads
 			Logger.Log.Debug ("Starting Inotify threads");
 			Inotify.Start ();
-
+	
 			// Test if the FileAdvise stuff is working: This will print a
 			// warning if not.  The actual advice calls will fail silently.
 			FileAdvise.TestAdvise ();
@@ -394,11 +457,17 @@ namespace Beagle.Daemon {
 
 			Logger.Log.Debug ("Ready to accept requests after {0}", 
 					 stopwatch);
-			
+		
 			// Start our event loop.
 			Logger.Log.Debug ("Starting main loop");
 			Application.Run ();
 
+#if ENABLE_WEBSVC
+			if (appServer != null) {
+			    	appServer.Stop(); 
+				appServer = null;
+			}
+#endif
 			Logger.Log.Debug ("Leaving BeagleDaemon.Main");
 
 			// Exiting will close the dbus connection, which
@@ -408,6 +477,12 @@ namespace Beagle.Daemon {
 
 		private static void OnShutdown ()
 		{
+#if ENABLE_WEBSVC
+			if (appServer != null) {
+			    	appServer.Stop(); 
+				appServer = null;
+			}
+#endif
 			// Stop our Inotify threads
 			Inotify.Stop ();
 
